unit AddEdit;

interface

uses
   Windows, SysUtils, Classes, Graphics, Forms, TNTForms, TntStdCtrls, TntButtons,
   Menus, TNTSysUtils, TNTClasses, StrUtils, Math, TntDialogs,
   TntMenus, Buttons, ExtCtrls, StdCtrls, Controls, Dialogs;

function isGUID(const ws: WideString): Boolean;

type
   DriveLetters = record
      Number: ShortInt;
      Letters: array of Char;
   end;
   VMID = record
      Name: WideString;
      ID: AnsiString;
   end;

type
   TfrmAddEdit = class(TTNTForm)
      rbVirtualBox: TRadioButton;
      rbQEMU: TRadioButton;
      pnlAll: TPanel;
      edtName: TTntEdit;
      edtExeParams: TTntEdit;
      edtWaitTime: TTntEdit;
      cmbWS: TTntComboBox;
      cmbPriority: TTntComboBox;
      lblWaitTime: TTntLabel;
      lblType: TTntLabel;
      lblExeParams: TTntLabel;
      lblFirstDrive: TTntLabel;
      lblName: TTntLabel;
      lblRun: TTntLabel;
      lblSecondDrive: TTntLabel;
      lblVMName: TTntLabel;
      lblPriority: TTntLabel;
      btnOK: TTntBitBtn;
      btnCancel: TTntBitBtn;
      lblVMPath: TTntLabel;
      lblMode: TTntLabel;
      cmbMode: TTntComboBox;
      edtVMPath: TTntEdit;
      btnBrowseForVM: TTntSpeedButton;
      odSearchVM: TTntOpenDialog;
      cmbFirstDrive: TTntComboBox;
      cmbSecondDrive: TTntComboBox;
      cmbVMName: TTntComboBox;
      procedure rbVirtualBoxClick(Sender: TObject);
      procedure rbQEMUClick(Sender: TObject);
      procedure edtWaitTimeKeyPress(Sender: TObject; var Key: Char);
      procedure btnOKClick(Sender: TObject);
      procedure FormShow(Sender: TObject);
      procedure cmbModeChange(Sender: TObject);
      procedure btnBrowseForVMClick(Sender: TObject);
      procedure edtExeParamsChange(Sender: TObject);
      procedure cmbDriveChange(Sender: TObject);
      procedure cmbVMNameDropDown(Sender: TObject);
      procedure cmbDriveDropDown(Sender: TObject);
      procedure TntFormCreate(Sender: TObject);
   private
      { Private declarations }
      meParams: Boolean;
   public
      { Public declarations }
      aDL: array of DriveLetters;
      VMIDs: array of VMID;
      ShowedThisSession: Boolean;
      procedure GetNamesAndIDs;
   end;

var
   frmAddEdit: TfrmAddEdit;

implementation

uses MainForm, InfoMess;

{$R *.dfm}

function isGUID(const ws: WideString): Boolean;
var
   i, l: Integer;
begin
   Result := False;
   l := Length(ws);
   if l <> 36 then
      Exit;
   for i := 1 to 36 do
      if i in [9, 14, 19, 24] then
      begin
         if ws[i] <> '-' then
            Exit;
      end
      else if ((ws[i] < '0') or (ws[i] > '9')) and ((ws[i] < 'A') or (ws[i] > 'F')) and ((ws[i] < 'a') or (ws[i] > 'f')) then
         Exit;
   Result := True;
end;

procedure TfrmAddEdit.rbVirtualBoxClick(Sender: TObject);
var
   p: Integer;
begin
   lblMode.Visible := True;
   cmbMode.Visible := True;
   if not meParams then
   begin
      edtExeParams.Text := '';
      meParams := False;
   end;
   if Visible then
      cmbMode.ItemIndex := 0;
   cmbModeChange(cmbMode);
   p := cmbWS.ItemIndex;
   cmbWS.Items[3] := 'FullScreen';
   cmbWS.ItemIndex := p;
   Height := Height + lblType.Top - lblName.Top;
   if Visible then
      rbVirtualBox.SetFocus;
end;

procedure TfrmAddEdit.rbQEMUClick(Sender: TObject);
var
   p: Integer;
begin
   lblMode.Visible := False;
   cmbMode.Visible := False;
   if not meParams then
   begin
      edtExeParams.Text := QemuDefaultParameters;
      meParams := False;
   end;
   cmbMode.ItemIndex := 2;
   cmbModeChange(cmbMode);
   p := cmbWS.ItemIndex;
   cmbWS.Items[3] := 'Hidden';
   cmbWS.ItemIndex := p;
   Height := Height - lblType.Top + lblName.Top;
   if Visible then
      rbQemu.SetFocus;
end;

procedure TfrmAddEdit.edtWaitTimeKeyPress(Sender: TObject; var Key: Char);
begin
   if not (Key in ['0'..'9', #8]) then
      Key := #0;
end;

procedure TfrmAddEdit.btnOKClick(Sender: TObject);
var
   i, p, l, cp, n1, n2, n3, a1: Integer;
   ws, wst: WideString;
begin
   ModalResult := mrNone;
   if Trim(edtName.Text) = '' then
   begin
      MessageBoxW(Handle, 'Please set a name !', 'Warning', MB_OK or MB_ICONWARNING);
      edtName.SetFocus;
      Exit;
   end;
   if Caption = 'Add' then
   begin
      i := 0;
      ws := WideLowerCase(Trim(edtName.Text));
      while i < Length(VMEntries) do
      begin
         if WideLowerCase(VMEntries[i].Name) = ws then
            Break;
         Inc(i);
      end;
      if i < Length(VMEntries) then
      begin
         MessageBoxW(Handle, 'An entry with this name already exists !', 'Warning', MB_OK or MB_ICONWARNING);
         edtName.SetFocus;
         Exit;
      end;
   end
   else
   begin
      i := 0;
      ws := WideLowerCase(Trim(edtName.Text));
      while i < Length(VMEntries) do
      begin
         if WideLowerCase(VMEntries[i].Name) = ws then
            Break;
         Inc(i);
      end;
      if (i < Length(VMEntries)) and (i <> frmMain.lvVMs.ItemIndex) then
      begin
         MessageBoxW(Handle, 'Another entry with this name already exists !', 'Warning', MB_OK or MB_ICONWARNING);
         edtName.SetFocus;
         Exit;
      end;
   end;
   if rbVirtualBox.Checked then
   begin
      case cmbMode.ItemIndex of
         1: if Trim(edtVMPath.Text) = '' then
            begin
               MessageBoxW(Handle, 'Please set the VM Path !', 'Warning', MB_OK or MB_ICONWARNING);
               edtVMPath.SetFocus;
               Exit;
            end
            else
            begin
               ws := WideChangeFileExt(WideExtractFileName(Trim(edtVMPath.Text)), '');
               GetNamesAndIDs;
               i := 0;
               while i <= High(VMIDs) do
               begin
                  if VMIDs[i].Name = ws then
                     Break;
                  Inc(i);
               end;
               if i > High(VMIDs) then
               begin
                  MessageBoxW(Handle, 'Could not find this VM in VirtualBox configuration files !'#13#10'Please make sure that VirtualBox is properly installed'#13#10'and the VM is properly registered in the VirtualBox Manager.', 'Warning', MB_OK or MB_ICONWARNING);
                  edtVMPath.SetFocus;
                  Exit;
               end;
            end;
         2: if (Trim(edtExeParams.Text) = '') then
            begin
               MessageBoxW(Handle, 'Please set the Exe parameters !', 'Warning', MB_OK or MB_ICONWARNING);
               edtExeParams.SetFocus;
               Exit;
            end
            else
            begin
               ws := Trim(edtExeParams.Text);
               p := Pos('--startvm "', ws);
               l := Length(ws);
               if (p > 0) and ((p + 11) < l) then
               begin
                  cp := PosEx('"', ws, p + 11);
                  if cp > 0 then
                     ws := Trim(Copy(ws, p + 11, cp - p - 11))
                  else
                     ws := '';
               end
               else
                  ws := '';
               if WideFileExists(ws) then
               begin
                  wst := WideChangeFileExt(WideExtractFileName(wst), '');
                  GetNamesAndIDs;
                  i := 0;
                  while i <= High(VMIDs) do
                  begin
                     if VMIDs[i].Name = wst then
                        Break;
                     Inc(i);
                  end;
                  if i > High(VMIDs) then
                     ws := '';
               end
               else
               begin
                  with frmMain.xmlGen do
                  begin
                     if Tag = 1 then
                        try
                           Active := True;
                           n1 := ChildNodes.IndexOf('VirtualBox');
                           if n1 > -1 then
                           begin
                              n2 := ChildNodes[n1].ChildNodes.IndexOf('Global');
                              if n2 > -1 then
                              begin
                                 n3 := ChildNodes[n1].ChildNodes[n2].ChildNodes.IndexOf('SystemProperties');
                                 if n3 > -1 then
                                 begin
                                    a1 := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].AttributeNodes.IndexOf('defaultMachineFolder');
                                    if a1 > -1 then
                                    begin
                                       wst := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].AttributeNodes[a1].Text;
                                       Replacebks(wst, Length(wst));
                                    end;
                                 end;
                              end;
                           end;
                        except
                        end;
                     Active := False;
                  end;

                  if (WideExtractFileName(ws) = ws) and (WideExtractFileExt(ws) <> '') and WideFileExists(wst + '\' + WideChangeFileExt(ws, '') + '\' + ws) then
                  else if (WideExtractFileName(ws) = ws) and (WideExtractFileExt(ws) = '') and WideFileExists(wst + '\' + ws + '\' + ws + '.vbox') then
                  else if isGUID(ws) then
                  begin
                     i := 0;
                     while i <= High(VMIDs) do
                     begin
                        if VMIDs[i].ID = ws then
                           Break;
                        Inc(i);
                     end;
                     if i > High(VMIDs) then
                        ws := '';
                  end
                  else
                     ws := '';
               end;
               if ws = '' then
               begin
                  MessageBoxW(Handle, 'Please set the Exe parameters correctly !'#13#10'Just use <<--startvm "Path_to_VM or GUID">>', 'Warning', MB_OK or MB_ICONWARNING);
                  edtExeParams.SetFocus;
                  Exit;
               end;
            end
         else if cmbVMName.ItemIndex = 0 then
         begin
            if cmbVMName.Items.Count > 1 then
            begin
               MessageBoxW(Handle, 'Please set a VM Name !', 'Warning', MB_OK or MB_ICONWARNING);
               cmbVMName.SetFocus;
               Exit;
            end
            else
            begin
               MessageBoxW(Handle, 'Please add a VM in VirtualBox Manager or solve the problem'#13#10'with VirtualBox installation so you can set a VM Name !', 'Warning', MB_OK or MB_ICONWARNING);
               cmbVMName.SetFocus;
               Exit;
            end;
         end;
      end;
   end
   else if Trim(edtExeParams.Text) = '' then
   begin
      MessageBoxW(Handle, 'Please set the Exe parameters !', 'Warning', MB_OK or MB_ICONWARNING);
      edtExeParams.SetFocus;
      Exit;
   end;
   if cmbFirstDrive.ItemIndex < 1 then
   begin
      if AddSecondDrive then
         MessageBoxW(Handle, 'Please choose the first drive !', 'Warning', MB_OK or MB_ICONWARNING)
      else
         MessageBoxW(Handle, 'Please choose the drive !', 'Warning', MB_OK or MB_ICONWARNING);
      cmbFirstDrive.SetFocus;
      Exit;
   end;
   if AddSecondDrive then
      if cmbFirstDrive.ItemIndex = cmbSecondDrive.ItemIndex then
      begin
         MessageBoxW(Handle, 'Please set a different second drive !', 'Warning', MB_OK or MB_ICONWARNING);
         cmbSecondDrive.SetFocus;
         Exit;
      end;
   i := StrToIntDef(Trim(edtWaitTime.Text), -1);
   if (i < 0) or (i > 20000) then
   begin
      MessageBoxW(Handle, 'Please set a proper value for the wait time (0..20000) !', 'Warning', MB_OK or MB_ICONWARNING);
      edtWaitTime.SetFocus;
      Exit;
   end;
   ModalResult := mrOK;
end;

procedure TfrmAddEdit.FormShow(Sender: TObject);
begin
   edtName.SetFocus;
   edtName.SelectAll;
end;

procedure TfrmAddEdit.cmbModeChange(Sender: TObject);
begin
   case cmbMode.ItemIndex of
      1:
         begin
            lblVMName.Visible := False;
            cmbVMName.Visible := False;
            lblVMPath.Visible := True;
            edtVMPath.Visible := True;
            btnBrowseForVM.Visible := True;
            lblExeParams.Visible := False;
            edtExeParams.Visible := False;
            if Visible then
            begin
               edtVMPath.SetFocus;
               edtVMPath.SelStart := Length(edtVMPath.Text);
               edtVMPath.SelLength := 0;
            end;
         end;
      2:
         begin
            lblVMName.Visible := False;
            cmbVMName.Visible := False;
            lblVMPath.Visible := False;
            edtVMPath.Visible := False;
            btnBrowseForVM.Visible := False;
            lblExeParams.Visible := True;
            edtExeParams.Visible := True;
            if Visible then
            begin
               edtExeParams.SetFocus;
               edtExeParams.SelStart := Length(edtExeParams.Text);
               edtExeParams.SelLength := 0;
            end;
         end;
   else
      begin
         lblVMName.Visible := True;
         cmbVMName.Visible := True;
         lblVMPath.Visible := False;
         edtVMPath.Visible := False;
         btnBrowseForVM.Visible := False;
         lblExeParams.Visible := False;
         edtExeParams.Visible := False;
         if Visible then
            cmbVMName.SetFocus;
      end;
   end;
end;

procedure TfrmAddEdit.btnBrowseForVMClick(Sender: TObject);
var
   FolderName: WideString;
   n1, n2, n3, a, i, l: Integer;
begin
   FolderName := WideExtractFilePath(odSearchVM.FileName);
   odSearchVM.FileName := '';
   if FolderName = '' then
   begin
      if odSearchVM.InitialDir = '' then
      begin

         with frmMain.xmlGen do
         begin
            if Tag = 1 then
               try
                  Active := True;
                  n1 := ChildNodes.IndexOf('VirtualBox');
                  if n1 > -1 then
                  begin
                     n2 := ChildNodes[n1].ChildNodes.IndexOf('Global');
                     if n2 > -1 then
                     begin
                        n3 := ChildNodes[n1].ChildNodes[n2].ChildNodes.IndexOf('SystemProperties');
                        if n3 > -1 then
                        begin
                           a := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].AttributeNodes.IndexOf('defaultMachineFolder');
                           if a > -1 then
                           begin
                              FolderName := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].AttributeNodes[a].Text;
                              l := Length(FolderName);
                              if l > 2 then
                              begin
                                 i := 1;
                                 while i <= l do
                                 begin
                                    if FolderName[i] = '/' then
                                       FolderName[i] := '\';
                                    Inc(i);
                                 end;
                              end;
                           end;
                        end;
                     end;
                  end;
               except
               end;
            Active := False;
         end;

         if WideDirectoryExists(FolderName) then
            odSearchVM.InitialDir := FolderName;
      end;
   end
   else
      odSearchVM.InitialDir := FolderName;

   if odSearchVM.Execute then
   begin
      edtVMPath.Text := odSearchVM.FileName;
      edtVMPath.SetFocus;
      edtVMPath.SelStart := Length(edtVMPath.Text);
      edtVMPath.SelLength := 0;
   end;
end;

procedure TfrmAddEdit.edtExeParamsChange(Sender: TObject);
begin
   meParams := True;
end;

procedure TfrmAddEdit.cmbDriveChange(Sender: TObject);
begin
   try
      if not rbQemu.Checked then
         if (Sender as TTntComboBox).ItemIndex > 0 then
            if (not DriveMessageShowed) and (not ShowedThisSession) then
            begin
               try
                  if frmInfoMess = nil then
                     Application.CreateForm(TfrmInfoMess, frmInfoMess);
                  with frmInfoMess do
                  begin
                     lblMess.Caption := 'It will add the drive to the first available port in the VirtualBox VM''s storage controller(s).'#13#10'If it will not find one available it will fail...'#13#10 +
                        #13#10'If you want to boot from this drive make sure that the available port'#13#10'is prior to other ports with HDDs and the VirtualBox VM is set to boot from HDD.';
                     ShowModal;
                     DriveMessageShowed := cbNoConfirm.Checked;
                  end;
                  ShowedThisSession := True;
               except
               end;
               try
                  if frmInfoMess <> nil then
                  begin
                     frmInfoMess.Free;
                     frmInfoMess := nil;
                  end;
               except
               end;
            end;
   except
   end;
end;

procedure TfrmAddEdit.GetNamesAndIDs;
var
   i, p1, l, n1, n2, n3: Integer;
   wn, wid: WideString;
begin
   SetLength(VMIDs, 0);
   with frmMain.xmlGen do
   begin
      if Tag = 1 then
         try
            Active := True;
            n1 := ChildNodes.IndexOf('VirtualBox');
            if n1 > -1 then
            begin
               n2 := ChildNodes[n1].ChildNodes.IndexOf('Global');
               if n2 > -1 then
               begin
                  n3 := ChildNodes[n1].ChildNodes[n2].ChildNodes.IndexOf('MachineRegistry');
                  if n3 > -1 then
                     for i := 0 to ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].ChildNodes.Count - 1 do
                        try
                           p1 := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].ChildNodes[i].AttributeNodes.IndexOf('src');
                           if p1 = -1 then
                              Continue;
                           wn := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].ChildNodes[i].AttributeNodes[p1].Text;
                           l := Length(wn);
                           if l < 9 then
                              Continue;
                           p1 := l;
                           while p1 > 0 do
                           begin
                              if (wn[p1] = '\') or (wn[p1] = '/') then
                                 Break;
                              Dec(p1);
                           end;
                           wn := WideChangeFileExt(Copy(wn, p1 + 1, l - p1), '');
                           p1 := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].ChildNodes[i].AttributeNodes.IndexOf('uuid');
                           if p1 = -1 then
                              Continue;
                           wid := ChildNodes[n1].ChildNodes[n2].ChildNodes[n3].ChildNodes[i].AttributeNodes[p1].Text;
                           l := Length(wid);
                           if l < 3 then
                              Continue;
                           SetLength(VMIDs, Length(VMIDs) + 1);
                           VMIDs[High(VMIDs)].Name := wn;
                           VMIDs[High(VMIDs)].ID := Copy(wid, 2, l - 2);
                        except
                        end;
               end;
            end;
         except
         end;
      Active := False;
   end;
end;

procedure TfrmAddEdit.cmbVMNameDropDown(Sender: TObject);
var
   i: Integer;
   t: TTNTStrings;
begin
   GetNamesAndIDs;
   try
      t := TTntStringList.Create;
      t.Add('None');
      for i := 0 to High(VMIDs) do
         t.Add('"' + VMIDs[i].Name + '"');
      if not cmbVMName.Items.Equals(t) then
      begin
         i := cmbVMName.ItemIndex;
         cmbVMName.Items.Assign(t);
         cmbVMName.ItemIndex := Min(cmbVMName.Items.Count - 1, i);
      end;
      t.Free;
   except
   end;
end;

procedure TfrmAddEdit.cmbDriveDropDown(Sender: TObject);
var
   i, j: Integer;
   sz: Double;
   hVolume, hDrive: THandle;
   dwBytesReturned, btsIO: DWORD;
   sdn: STORAGE_DEVICE_NUMBER;
   ErrorMode: Word;
   ws: WideString;
   csz, mu: AnsiString;
   RawMBR: array[0..511] of Byte;
   CanRead: Boolean;
   t: TTNTStrings;

   procedure Round3;
   var
      k, n: Integer;
   begin
      k := 0;
      if sz < 100 then
      begin
         while sz < 1000 do
         begin
            sz := sz * 10;
            Inc(k);
         end;
         Dec(k);
         csz := IntToStr(Round(sz / 10));
         if k <= 2 then
            Insert(DecimalSeparator, csz, 4 - k)
         else
         begin
            for n := 1 to k - 2 do
               csz := '0' + csz;
            Insert(DecimalSeparator, csz, 2);
         end;
      end
      else if sz > 1000 then
      begin
         while sz > 100 do
         begin
            sz := sz / 10;
            Inc(k);
         end;
         Dec(k);
         csz := IntToStr(Round(sz * 10));
         for n := 1 to k do
            csz := csz + '0';
      end
      else
         csz := IntToStr(Round(sz));
   end;

begin
   t := TTntStringList.Create;
   t.Add('None');
   ErrorMode := SetErrorMode(SEM_FailCriticalErrors);
   SetLength(aDL, 1);
   aDl[0].Number := -1;
   for i := Byte('A') to Byte('Z') do
   begin
      try
         case GetDriveType(PChar(Char(i) + ':\')) of
            DRIVE_REMOVABLE, DRIVE_FIXED:
               begin
                  try
                     hVolume := CreateFile(PAnsiChar('\\.\' + Char(i) + ':'), 0, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
                  except
                     hVolume := INVALID_HANDLE_VALUE;
                  end;
                  if hVolume <> INVALID_HANDLE_VALUE then
                  begin
                     dwBytesReturned := 0;
                     try
                        if DeviceIoControl(hVolume, IOCTL_STORAGE_GET_DEVICE_NUMBER, nil, 0, @sdn, sizeof(sdn), dwBytesReturned, nil) then
                           if sdn.DeviceNumber <> OSDrive then
                           begin
                              try
                                 hDrive := CreateFile(PAnsiChar('\\.\PHYSICALDRIVE' + IntToStr(sdn.DeviceNumber)), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
                              except
                                 hDrive := INVALID_HANDLE_VALUE;
                              end;
                              if hDrive <> INVALID_HANDLE_VALUE then
                              begin
                                 try
                                    SetFilePointer(hDrive, 512, nil, FILE_BEGIN);
                                    CanRead := ReadFile(hDrive, RawMBR[0], 512, btsIO, nil);
                                 except
                                    CanRead := False;
                                 end;
                                 try
                                    CloseHandle(hDrive);
                                 except
                                 end;
                                 if CanRead then
                                 begin
                                    j := 0;
                                    while j <= High(aDL) do
                                    begin
                                       if Cardinal(aDL[j].Number) = sdn.DeviceNumber then
                                          Break;
                                       Inc(j);
                                    end;
                                    if j > High(aDL) then
                                    begin
                                       SetLength(adl, Length(adl) + 1);
                                       aDL[j].Number := sdn.DeviceNumber;
                                    end;
                                    SetLength(aDL[j].Letters, Length(aDL[j].Letters) + 1);
                                    aDL[j].Letters[High(aDL[j].Letters)] := Char(i);
                                 end;
                              end;
                           end;
                     except
                     end;
                     CloseHandle(hVolume);
                  end;
               end
         else
            Continue;
         end;
      except
      end;
   end;
   for i := 0 to MAX_IDE_DRIVES - 1 do
   begin
      if i = OSDrive then
         Continue;
      j := 0;
      while j <= High(aDL) do
      begin
         if aDL[j].Number = i then
            Break;
         Inc(j);
      end;
      if j > High(aDL) then
      begin
         try
            hDrive := CreateFile(PAnsiChar('\\.\PHYSICALDRIVE' + IntToStr(i)), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
         except
            hDrive := INVALID_HANDLE_VALUE;
         end;
         if hDrive <> INVALID_HANDLE_VALUE then
         begin
            try
               SetFilePointer(hDrive, 512, nil, FILE_BEGIN);
               CanRead := ReadFile(hDrive, RawMBR[0], 512, btsIO, nil);
            except
               CanRead := False;
            end;
            try
               CloseHandle(hDrive);
            except
            end;
            if CanRead then
            begin
               SetLength(aDL, Length(aDL) + 1);
               aDL[High(aDL)].Number := i;
            end;
         end;
      end;
   end;
   if Length(aDl) > 1 then
   begin
      for i := 1 to High(aDL) do
      begin
         ws := GetDriveVendorAndProductID(aDL[i].Number);
         sz := GetDriveSize(aDL[i].Number) / 1073741824;
         if sz < 0 then
         else
         begin
            if sz < 1 then
            begin
               sz := sz * 1024;
               mu := ' MB';
            end
            else if sz > 1000 then
            begin
               sz := sz / 1024;
               mu := ' TB';
            end
            else
               mu := ' GB';
            Round3;
            ws := ws + ', ' + csz + mu;
         end;
         ws := ws + ',  [';
         if Length(aDl[i].Letters) > 0 then
         begin
            j := 0;
            while j < High(aDl[i].Letters) do
            begin
               ws := ws + aDl[i].Letters[j] + ', ';
               Inc(j);
            end;
            ws := ws + aDl[i].Letters[j];
         end;
         ws := ws + ']';
         t.Add(ws);
      end;
   end;
   SetErrorMode(ErrorMode);
   try
      if not (Sender as TTntComboBox).Items.Equals(t) then
      begin
         i := (Sender as TTntComboBox).ItemIndex;
         (Sender as TTntComboBox).Items.Assign(t);
         (Sender as TTntComboBox).ItemIndex := Min((Sender as TTntComboBox).Items.Count - 1, i);
      end;
   except
   end;
   t.Free;
end;

procedure TfrmAddEdit.TntFormCreate(Sender: TObject);
begin
   if not AddSecondDrive then
   begin
      lblSecondDrive.Visible := False;
      cmbSecondDrive.Visible := False;
      lblFirstDrive.Caption := 'Drive to add:';
      lblFirstDrive.Anchors := [akTop];
      cmbFirstDrive.Anchors := [akTop];
      lblVMPath.Anchors := [akTop];
      lblVMName.Anchors := [akTop];
      lblExeParams.Anchors := [akTop];
      cmbVMName.Anchors := [akTop];
      btnBrowseForVM.Anchors := [akTop];
      edtVMPath.Anchors := [akTop];
      edtExeParams.Anchors := [akTop];
      Height := Height - lblType.Top + lblName.Top;
      lblFirstDrive.Anchors := [akLeft, akBottom];
      cmbFirstDrive.Anchors := [akRight, akBottom];
      lblVMPath.Anchors := [akLeft, akBottom];
      lblVMName.Anchors := [akLeft, akBottom];
      lblExeParams.Anchors := [akLeft, akBottom];
      cmbVMName.Anchors := [akRight, akBottom];
      btnBrowseForVM.Anchors := [akRight, akBottom];
      edtVMPath.Anchors := [akRight, akBottom];
      edtExeParams.Anchors := [akRight, akBottom];
      Width := Width + lblWaitTime.Width - lblSecondDrive.Width;
   end;
   ShowedThisSession := False;
end;

end.

