unit ProcessViewer;

interface

uses
   Windows, SysUtils, TNTClasses, TLHelp32;

type
   TProcessInfo = record
      Caption: WideString;
      Handle: DWord;
      PClass: WideString;
   end;

var

   CaptionList, HandleList, ClassList: TTNTStringList;
   ProcessInfo: array of TProcessInfo;

function EnumWindowsProc(hWnd: HWND; lParam: LPARAM): Bool; stdcall;
procedure GetProcessList;
function GetFileNameFromHandleW(Handle: hwnd): WideString;
function GetFileNameAndThreadFromHandleW(Handle: hwnd; var ThreadID: THandle): WideString;

implementation

procedure GetProcessList;
var
   i, Len: Integer;
begin
   HandleList.Clear;
   ClassList.Clear;
   CaptionList.Clear;
   EnumWindows(@EnumWindowsProc, 0);
   Len := HandleList.Count;
   SetLength(ProcessInfo, Len);
   for i := 0 to Len - 1 do
   begin
      with ProcessInfo[i] do
      begin
         Caption := CaptionList[i];
         Handle := StrToInt64(HandleList[i]);
         PClass := ClassList[i];
      end;
   end;
end;

function EnumWindowsProc(hWnd: HWND; lParam: LPARAM): Bool;
var
   Capt, Cla: array[0..255] of WideChar;
begin
   GetWindowTextW(hWnd, Capt, 255);
   GetClassNameW(hwnd, Cla, 255);
   HandleList.Add(IntToStr(HWnd));
   ClassList.Add(Cla);
   CaptionList.Add(Capt);
   Result := True;
end;

function GetFileNameFromHandleW(Handle: hwnd): WideString;
var
   PID: DWord;
   aSnapShotHandle: THandle;
   ContinueLoop: Boolean;
   aProcessEntry32: TProcessEntry32W;
begin
   GetWindowThreadProcessID(Handle, @PID);
   aSnapShotHandle := CreateToolHelp32SnapShot(TH32CS_SNAPPROCESS, 0);
   aProcessEntry32.dwSize := SizeOf(aProcessEntry32);
   ContinueLoop := Process32FirstW(aSnapShotHandle, aProcessEntry32);
   while Integer(ContinueLoop) <> 0 do
   begin
      if aProcessEntry32.th32ProcessID = PID then
      begin
         Result := WideLowerCase(aProcessEntry32.szExeFile);
         Break;
      end;
      ContinueLoop := Process32NextW(aSnapShotHandle, aProcessEntry32);
   end;
   CloseHandle(aSnapShotHandle);
end;

function GetFileNameAndThreadFromHandleW(Handle: hwnd; var ThreadID: THandle): WideString;
var
   PID: DWord;
   aSnapShotHandle: THandle;
   ContinueLoop: Boolean;
   aProcessEntry32: TProcessEntry32W;
begin
   GetWindowThreadProcessID(Handle, @PID);
   aSnapShotHandle := CreateToolHelp32SnapShot(TH32CS_SNAPPROCESS, 0);
   aProcessEntry32.dwSize := SizeOf(aProcessEntry32);
   ContinueLoop := Process32FirstW(aSnapShotHandle, aProcessEntry32);
   while Integer(ContinueLoop) <> 0 do
   begin
      if aProcessEntry32.th32ProcessID = PID then
      begin
         Result := WideLowerCase(aProcessEntry32.szExeFile);
         ThreadID := aProcessEntry32.th32ProcessID;
         Break;
      end;
      ContinueLoop := Process32NextW(aSnapShotHandle, aProcessEntry32);
   end;
   CloseHandle(aSnapShotHandle);
end;

initialization
   HandleList := TTNTStringList.Create;
   ClassList := TTNTStringList.Create;
   CaptionList := TTNTStringList.Create;

finalization
   HandleList.Free;
   ClassList.Free;
   CaptionList.Free;

end.

